///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Author: JT Cook
//
// Last Modified: 01.01.09
//
// Description: A simple bowling game for the 16-bit XGS that supports NTSC, requires game controller
//
// Required Files: XGS_PIC_SYSTEM_V010.c/h
//				   XGS_PIC_GFX_DRV_V010.c/h
//				   XGS_PIC_GAMEPAD_DRV_V010.c/h
//				   XGS_PIC_NTSC_TILE1_V010.s/h - if you are using the TV
// 				   XGS_PIC_VGA_TILE1_V010.s/h - if you are using VGA (not yet finished)
//
//
// Todo:
// - add sound
// - finish VGA (add colors for VGA for sprite)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define XGS //comment out if you are using the simulator
#define VIDEO_NTSC //comment out for VGA mode, also uncomment out the VGA tile driver

#define ball_move_speed 2 //how fast ball moves up and down lane

#define font_offset 6 //where the font tiles starts
#define font_start 33 //where in the ascii chart the font starts
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// Include main system header
#include "XGS_PIC_SYSTEM_V010.h"

// Include graphics drivers
#include "XGS_PIC_GFX_DRV_V010.h"

//select video driver to include
 #include "XGS_PIC_NTSC_TILE1_V010.h"
// #include "XGS_PIC_VGA_TILE1_V010.h"

// Include the gamepad driver
#include "XGS_PIC_GAMEPAD_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned char g_Palettes[MAX_PALETTE_SIZE]; 	// Different colors
Sprite g_Sprites[MAX_SPRITES];					// Max sprites

// The TILE map buffer must be allocated into far memory space because 
// it is so large it wont fit into near 
unsigned char g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far));

// Character set as 8x8 with 1 byte per pixel
unsigned char g_TileData[] __attribute__((far)) = {

#ifdef  VIDEO_NTSC
	//#define TV_BLACK   	(0x4F)
    #define RW1 0x4F //floor
    #define RW2 0xFF //wall
    #define PC1 0xFF //pin solid
    #define PC2 0x44 //red pin stripe
    #define BB1 NTSC_BLUE //bowling ball
    #define FC1 0xFF //font color
#else
    //white 0x3F, black 0x00, red 0x03, blue 0x30, green 0x0C
    #define RW1 0x00 //floor
    #define RW2 0x3F //wall
    #define PC1 0x3F //pin solid
    #define PC2 0x03 //red pin stripe
    #define BB1 VGA_BLUE //bowling ball
    #define FC1 0x3F //font color

#endif
    //Tile #0 - Floor
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    
    
    //Tile #1 - wall
	RW2, RW2, RW2, RW2, RW2, RW2, RW2, RW2, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW2, RW2, RW2, RW2, RW2, RW2, RW2, RW2, 

    //Tile #2 - pin
	RW1, RW1, RW1, PC1, PC1, RW1, RW1, RW1, 
	RW1, RW1, PC1, PC1, PC1, PC1, RW1, RW1, 
    RW1, RW1, PC1, PC1, PC1, PC1, RW1, RW1, 
	RW1, RW1, RW1, PC1, PC1, RW1, RW1, RW1, 
    RW1, RW1, PC1, PC2, PC2, PC1, RW1, RW1, 
	RW1, PC1, PC1, PC1, PC1, PC1, PC1, RW1, 
    RW1, PC1, PC1, PC1, PC1, PC1, PC1, RW1, 
	RW1, RW1, PC1, PC1, PC1, PC1, RW1, RW1, 

    //Tile #3 - bowling ball
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, BB1, BB1, BB1, BB1, RW1, RW1, 
	RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
    RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
	RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
    RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
	RW1, RW1, BB1, BB1, BB1, BB1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
/*
    //Tile #4 - gray 4
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
*/
//Solid block
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,

//clear character, space
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,

//starts with tile 5
////////////// Punctuation 
// !
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// "
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// #
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
1, 1, 1, 1, 1, 1, 1, 1,
0, 1, 1, 0, 0, 1, 1, 0,
1, 1, 1, 1, 1, 1, 1, 1,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// $
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// %
0, 1, 1, 0, 0, 0, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// &
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 0, 1, 1, 1,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 1, 1,
0, 0, 0, 0, 0, 0, 0, 0,


// '
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// (
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// )
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// *
0, 0, 0, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
1, 1, 1, 1, 1, 1, 1, 1,
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// +
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// ,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,


// -
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// .
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// /
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 1, 1,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,




/////////// LETTERS 0-9 in C64 font
// 0
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 1, 1, 1, 0,
0, 1, 1, 1, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 1
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 2
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 3
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 4
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 1, 0,
0, 0, 0, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 1, 1,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 5
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 6
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 7
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 8
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// 9
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,

//////////// More punctuation
// :
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// ;
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,


// <
0, 0, 0, 0, 1, 1, 1, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// =
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// >
0, 1, 1, 1, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 1, 1, 1, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// ?
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,

// @
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 1, 1, 1, 0,
0, 1, 1, 0, 1, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,



/////////// LETTERS A-Z in C64 font
// A
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// B
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// C
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// D
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// E
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// F
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// G
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// H
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// I
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// J
0, 0, 0, 1, 1, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 0, 1, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// K
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 1, 0, 0, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// L
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// M
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 1, 0, 1, 1, 1,
0, 1, 1, 1, 1, 1, 1, 1,
0, 1, 1, 0, 1, 0, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 0, 0, 0, 0, 0, 0, 0,


// N
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 1, 1, 0, 1, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// O
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// P
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// Q
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// R
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 1, 1, 1, 0, 0,
0, 1, 1, 1, 1, 0, 0, 0,
0, 1, 1, 0, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// S
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// T
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// U
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// V
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// W
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 1, 1, 0, 1, 0, 1, 1,
0, 1, 1, 1, 1, 1, 1, 1,
0, 1, 1, 1, 0, 1, 1, 1,
0, 1, 1, 0, 0, 0, 1, 1,
0, 0, 0, 0, 0, 0, 0, 0,


// X
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// Y
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 1, 1, 0, 0, 1, 1, 0,
0, 0, 1, 1, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 0, 0, 0, 0, 0, 0,


// Z
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 1, 1, 0,
0, 0, 0, 0, 1, 1, 0, 0,
0, 0, 0, 1, 1, 0, 0, 0,
0, 0, 1, 1, 0, 0, 0, 0,
0, 1, 1, 0, 0, 0, 0, 0,
0, 1, 1, 1, 1, 1, 1, 0,
0, 0, 0, 0, 0, 0, 0, 0,
};

//Pin patern, for redrawing the pins
unsigned char pins [] = {
   
   0, 0, 0, 2, 0,     
   0, 0, 2, 0, 0,      
   0, 2, 0, 2, 0,     
   2, 0, 2, 0, 0,   
   0, 2, 0, 2, 0,     
   0, 0, 2, 0, 0,      
   0, 0, 0, 2, 0,     
};   

// Sprite bitmap data 8x8 pixels with 0 = transparent value
unsigned char g_SpriteData[] __attribute__((far)) = {
	#undef  RW1
	#define RW1 0x00 //transparent
    //sprite #0 - bowling ball
    RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	RW1, RW1, BB1, BB1, BB1, BB1, RW1, RW1, 
	RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
    RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
	RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
    RW1, BB1, BB1, BB1, BB1, BB1, BB1, RW1, 
	RW1, RW1, BB1, BB1, BB1, BB1, RW1, RW1, 
	RW1, RW1, RW1, RW1, RW1, RW1, RW1, RW1, 
	#ifdef VIDEO_NTSC

   	 #define BH1 0x42 // bowler's hair
     #define BS1 0xC4 // bowler's skin
     #define BS2 0x70 //bowler's shirt
     #define BP1 0x7F //bowler's pants
     #define BB2 NTSC_BLACK //drawing between bowler's arm
	#else
	#endif
    //sprite #1 - bowler frame 1, upper left
    RW1,RW1,RW1,RW1,RW1,RW1,BH1,BH1,
	RW1,RW1,RW1,RW1,RW1,RW1,BH1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,BH1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS1,
	RW1,RW1,RW1,RW1,RW1,BS2,BS2,BS2,
	RW1,RW1,RW1,RW1,BS1,BS2,BS2,BS2,
	RW1,RW1,RW1,RW1,BS1,BS1,BS2,BS2,
	RW1,RW1,RW1,BS1,BS1,BB2,BS2,BS2,
    //sprite #2 - bowler frame 1, upper right
    BH1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    //sprite #3 - bowler frame 1, lower left
	RW1,RW1,RW1,BS1,BS1,BB2,BS2,BS2,
	RW1,RW1,BB1,BS1,BS1,BB1,BS2,BS2,
	RW1,BB1,BB1,BS1,BS1,BB1,BB1,BP1,
	RW1,BB1,BB1,BB1,BB1,BB1,BB1,BP1,
	RW1,BB1,BB1,BB1,BB1,BB1,BB1,RW1,
	RW1,BB1,BB1,BB1,BB1,BB1,BB1,RW1,
	RW1,RW1,BB1,BB1,BB1,BB1,RW1,RW1,
	RW1,RW1,RW1,RW1,BP1,BP1,RW1,RW1,
    //sprite #4 - bowler frame 1, lower right
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BS2,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
    BP1,BP1,RW1,RW1,RW1,RW1,RW1,RW1,
    BP1,BP1,BP1,RW1,RW1,RW1,RW1,RW1,
	RW1,BP1,BP1,BP1,RW1,RW1,RW1,RW1,
	RW1,RW1,BP1,BP1,RW1,RW1,RW1,RW1,
	RW1,RW1,BP1,BP1,RW1,RW1,RW1,RW1,
	RW1,RW1,BP1,BP1,BP1,RW1,RW1,RW1,
    //////
	//sprite #5 - bowler frame 2, upper left
	//RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BH1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BH1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BH1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS2,
	RW1,RW1,RW1,RW1,RW1,RW1,BS2,BS2,
	RW1,RW1,RW1,RW1,RW1,RW1,BS2,BS2,
	//sprite #6 - bowler frame 2, upper right
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	BH1,BH1,RW1,RW1,RW1,RW1,RW1,RW1,
	BS1,BS1,RW1,RW1,RW1,RW1,RW1,RW1,
	BS1,BS1,RW1,RW1,RW1,RW1,RW1,RW1,
	BS1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	BS2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	BS2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	BB2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	//srpite #7 - bowler frame 2, lower left
	RW1,RW1,RW1,RW1,RW1,RW1,BS1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,BS1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,BS1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS1,
	RW1,RW1,RW1,RW1,RW1,RW1,BP1,BB1,
	RW1,RW1,RW1,RW1,BP1,BP1,BP1,BB1,
	RW1,RW1,RW1,BP1,BP1,RW1,RW1,BB1,
	RW1,RW1,RW1,RW1,BP1,BP1,RW1,RW1,
	//sprite #8 - bowler frame 2, lower right
	BB2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	BB2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	BB2,BS2,RW1,RW1,RW1,RW1,RW1,RW1,
	BB2,BB2,BB1,BB1,RW1,RW1,RW1,RW1,
	BS1,BS1,BB1,BB1,BB1,RW1,RW1,RW1,
	BS1,BS1,BB1,BB1,BB1,RW1,RW1,RW1,
	BB1,BB1,BB1,BB1,BB1,RW1,RW1,RW1,	
	BB1,BB1,BB1,BB1,BP1,RW1,RW1,RW1,
	/////
	//sprite #9 - bowler frame 3, upper left
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS2,
	//sprite #10 - bowler frame 3, upper right
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	BH1,BH1,BH1,RW1,RW1,RW1,RW1,RW1,
	BH1,BS1,BS1,RW1,RW1,RW1,RW1,RW1,
	BH1,BS1,BS1,RW1,RW1,RW1,RW1,RW1,
	RW1,BS1,RW1,RW1,RW1,RW1,RW1,RW1,
	BS2,BS2,BS2,RW1,RW1,RW1,RW1,RW1,
	BS2,BS2,BS2,RW1,RW1,RW1,RW1,RW1,
    //sprite #11 - bowler frame 3, lower left
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS2,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS2,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,BS2,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,RW1,RW1,RW1,BP1,BP1,
	RW1,RW1,RW1,RW1,BP1,BP1,BP1,BP1,
	RW1,RW1,RW1,RW1,BP1,BP1,RW1,RW1,
	//sprite #12 - bowler frame 3, lower right
	BS2,BS1,BS2,RW1,RW1,RW1,RW1,RW1,
	BS1,BS1,BS2,RW1,RW1,RW1,RW1,RW1,
	BS1,BS1,BB2,RW1,RW1,RW1,RW1,RW1,
	BS1,BS1,BS1,BS1,BP1,RW1,RW1,RW1,
	BP1,BP1,BS1,BS1,BS1,BS1,RW1,RW1,
	BP1,BP1,BP1,BP1,BS1,BS1,RW1,RW1,
	BP1,RW1,RW1,BP1,RW1,RW1,RW1,RW1,
	RW1,RW1,RW1,BP1,BP1,RW1,RW1,RW1,
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void init_game(); //setup game
void main_loop(); //main chunk of game
void check_pin(int x, int y); //check adjacent pins that are hit
void bowl_ball(int x, int y); //roll the ball down the lane
void draw_pins(); //draws the pins on the play field
void new_game(); //start a new game
void draw_title_screen(); //draws the title screen
void draw_game_screen(int max_players); //draws the game play screen (with bowling alley and score board)
void print_score(int player); //prints the current score
void draw_frame_score(int player, int throw); //draws score in the frame
void draw_current_player(int player, char on_off); //toggle current player on/off
void animate_bowler(); //do bowler animation
//int Add_score(int *g_score); //counts the current score

unsigned char GamepadValue,GamepadValueOld;
int i;
int ball_speed_counter, ball_dir, player_on_off, player_flash_ctr;
int ball_x, ball_y, ball_x_old, ball_y_old;
#define Lane_Height 9 //how tall the lane is
#define Lane_Start 13 //start of upper part of lane
#define Lane_End 21 //bottom part of the lane

char Frame_Count[11*3]; //score for frames
char Pin_count[2]; //number of pins hit per turn
char Player_turn, Player_frame; //which part of the frame, which frame player is in
int max_players, player; //current player
int game_state=0; //used inside main loop


// Our main start location
int main (void)
{
    init_game(); //setup game
    main_loop(); //main chunk of game
}

void init_game() //setup game
{
    #ifdef VIDEO_NTSC
	 // Always call ConfigureClock first
	  SYS_ConfigureClock(FCY_NTSC);
    #else
	  // Always call ConfigureClock first
	  SYS_ConfigureClock(FCY_VGA);
    #endif
	// Init the gamepad
	Gamepad_Init();
	// Init the graphics system	
    GFX_InitTile(g_TileData, &g_TileMap[0], g_Palettes, g_Sprites);
	// Loop through and give everything their default color
	for(i = 0; i < MAX_PALETTE_SIZE; i++)
		g_Palettes[i] = i;
    #ifdef VIDEO_NTSC
	  // Set palette 0 to blue for text background, white for fon
	  g_Palettes[0] = NTSC_BLACK;
	  g_Palettes[1] = NTSC_WHITE;
	  // Start up our display interrupt
	  GFX_StartDrawing(SCREEN_TYPE_NTSC);
    #else
	  // Set palette 0 to blue for text background, white for fon
	  g_Palettes[0] = VGA_BLACK;
	  g_Palettes[1] = VGA_WHITE;
	  // Start up our display interrupt
	  GFX_StartDrawing(SCREEN_TYPE_VGA);
    #endif

	// bowling ball sprite
	g_Sprites[0].State = SPRITE_STATE_ON;
	g_Sprites[0].Attribute = 0;
	g_Sprites[0].X = 88;
	g_Sprites[0].Y = 104;
	g_Sprites[0].PixelData = g_SpriteData;


}

void main_loop() //main chunk of game
{
#ifdef XGS
	while(1)
#endif
	{
		// Wait until we are in v-sync to do any processing
		WAIT_FOR_VSYNC_START();
		// Read the gamepad
		GamepadValue = Gamepad_Read(0);

        switch(game_state)
		{
          case 0: //draw title screen
			draw_title_screen();
            game_state=1;
            ball_x=4;
            ball_y=13;
            ball_dir=0; //cursor pos
			g_Sprites[0].X = (ball_x+1)*8;
			g_Sprites[0].Y = (ball_y+1)*8;
          break;
          case 1: //game select screen
		  {
            //cursor at X-4, Y-13,14,15
            //hit select to move cursor
			if((GamepadValue & GAMEPAD_SELECT)  && GamepadValueOld==0)
            {
				ball_dir++;
                if(ball_dir>2)
                   ball_dir=0;
                ball_y=13+ball_dir; //new cursor position

				g_Sprites[0].State = SPRITE_STATE_ON;
				g_Sprites[0].X = (ball_x+1)*8;
				g_Sprites[0].Y = (ball_y+1)*8;
               
			}
            //if we hit a button, start a new game
			if((GamepadValue & GAMEPAD_ACTION1 || GamepadValue & GAMEPAD_ACTION2 || GamepadValue & GAMEPAD_START)  && GamepadValueOld==0)
			{
                max_players=ball_dir; //total players
				player=0; //starting player
   			    new_game(); //setup a new game     
    			draw_game_screen(ball_dir+1); //draw game screen with number of players selected
				game_state=2; //start game
			}
		  }
          break;
          case 2: //main game
          {
		    player_flash_ctr++;
		    if( player_flash_ctr>7)
			{
				player_flash_ctr=0;
				player_on_off^=1;
				if(max_players==0)
				   player_on_off=1;
				draw_current_player(player,player_on_off); //toggle current player on/off
			}
        	//delay for moving ball
        	ball_speed_counter--;
        	//move ball location (before player hits button)
       	    if(ball_speed_counter<1)
        	{
         		ball_speed_counter=ball_move_speed;
         		if(ball_dir==0)
         		{
           			ball_y++;
           			if(ball_y>Lane_End-1)
			 		ball_dir^=1;
		 		}
         		else
         		{
           			ball_y--;
           			if(ball_y<Lane_Start+1)
            			ball_dir^=1;
		 		}
				g_Sprites[0].X = (ball_x+1)*8;
				g_Sprites[0].Y = (ball_y+1)*8;
			}
		
			// Roll the ball
			if((GamepadValue & GAMEPAD_ACTION1 || GamepadValue & GAMEPAD_ACTION2)  && GamepadValueOld==0)
			{
				g_Sprites[0].State = SPRITE_STATE_OFF;	
				//draw bowler
				//upper left
				g_Sprites[1].State = SPRITE_STATE_ON;
				g_Sprites[1].Attribute = 0;
				g_Sprites[1].X = (ball_x-1+1)*8;
				g_Sprites[1].Y = (ball_y-1+1)*8;;
				g_Sprites[1].PixelData = &g_SpriteData[64];
				//upper right
				g_Sprites[2].State = SPRITE_STATE_ON;
				g_Sprites[2].Attribute = 0;
				g_Sprites[2].X = (ball_x+1)*8;
				g_Sprites[2].Y = (ball_y-1+1)*8;;
				g_Sprites[2].PixelData = &g_SpriteData[64*2];
				//lower left
				g_Sprites[3].State = SPRITE_STATE_ON;
				g_Sprites[3].Attribute = 0;
				g_Sprites[3].X = (ball_x-1+1)*8;
				g_Sprites[3].Y = (ball_y+1)*8;;
				g_Sprites[3].PixelData = &g_SpriteData[64*3];
				//lower right
				g_Sprites[4].State = SPRITE_STATE_ON;
				g_Sprites[4].Attribute = 0;
				g_Sprites[4].X = (ball_x+1)*8;
				g_Sprites[4].Y = (ball_y+1)*8;;
				g_Sprites[4].PixelData = &g_SpriteData[64*4];
				animate_bowler();
				g_Sprites[0].State = SPRITE_STATE_ON;
				draw_current_player(player,1); //toggle current player on/off
			  	bowl_ball(ball_x,ball_y);
				g_Sprites[1].State = SPRITE_STATE_OFF;
				g_Sprites[2].State = SPRITE_STATE_OFF;
				g_Sprites[3].State = SPRITE_STATE_OFF;
				g_Sprites[4].State = SPRITE_STATE_OFF;
			}
		  }
          break;
          case 3: //game over, wait for player to hit button
          {
			if((GamepadValue & GAMEPAD_ACTION1 || GamepadValue & GAMEPAD_ACTION2 || GamepadValue & GAMEPAD_START)  && GamepadValueOld==0)
			{
			  	game_state=0;
			}
		  }
		}
        GamepadValueOld=GamepadValue; //grab last button press
		// Wait here till we are done with v-sync
		WAIT_FOR_VSYNC_END();
	}

	return 0;
}
//////////////////////////////////////////////////////////////
//screen helpers

//draws a digit on the screen
void print_char(int x, int y, char digit)
{
 g_TileMap[y*SCREEN_TILE_MAP_WIDTH+x]=digit-font_start+font_offset; //draw digit
}
//prints a string
void print_string(int x, int y, char *string)
{
 int n, offset;
 offset=font_offset-font_start;
 for(n=0;n<20;n++)
 {
   //make sure we are only printing characters that we have graphics for
   if(string[n]>32 && string[n]<91)
    g_TileMap[y*SCREEN_TILE_MAP_WIDTH+x]=string[n]+offset; //draw digit
   else if(string[n]==0)
    return;
   x++;
 }
}
//fills the screen with a select character
void fill_screen(char digit)
{
 int n;
 for(n=0;n<(SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT);n++)
  g_TileMap[n]=digit; //draw ball
}

//////////////////////////////////////////////////////////////
//Gameplay stuff

void bowl_ball(int x, int y)
{
     int sprite_x, x_cntr;
     int player_offset;
     player_offset=player*11; //offset for player
     sprite_x=(x+1)*8;
     x_cntr=0;
     WAIT_FOR_VSYNC_END();
     while(x<18)
     {
        WAIT_FOR_VSYNC_START();

          sprite_x+=2;
		  x_cntr+=2;
		  g_Sprites[0].X = sprite_x;
		  //check tile for pins
		  if(x_cntr>7)
		  {
		  	x++;
          	check_pin(x,y); //check to see if we hit any pins
			x_cntr-=8;
		  }
        WAIT_FOR_VSYNC_END();
     }
     //if we have rolled two balls for that frame, handle it
     if(Player_frame<9) // we are not at the 10th frame yet
     {
      //update score box
      draw_frame_score(player, Player_turn);
      //strike!
      if(Pin_count[0]==10)
        Player_turn++;
      //add score
      Frame_Count[Player_frame+player_offset]=Pin_count[0];
      print_score(player);
      Player_turn++;
      //we rolled the ball, new turn
      if(Player_turn>1)
      {
       //update score box
       draw_frame_score(player, Player_turn);

       //add the player's score            
       Frame_Count[Player_frame+player_offset]|=((Pin_count[1]&0x0F)<<4);
       //reset the scrore for the frame
       Pin_count[0]=0;
       Pin_count[1]=0;
       Player_turn=0;
       draw_pins(); //new frame, redraw the pins
       print_score(player); //draws the current score
	   
	   player++; //go to next player
	   //go back to first player, advance a frame
	   if(player>max_players)
	   {
        Player_frame++;
        player=0;
       }   
      }
	 }
     //handle the 10th frame
     else
	 {
      if(Player_turn==0  && Player_frame==9)
      {
       //strike
       if(Pin_count[0]==10)
        draw_pins(); // redraw the pins
       
       //update score box
       draw_frame_score(player, Player_turn);

	   Frame_Count[Player_frame+player_offset]=Pin_count[0];
       print_score(player);
       Player_turn++;
	  }
	  else if(Player_turn==1  && Player_frame==9)
	  {	
       //update score box
       draw_frame_score(player, Player_turn);

       //add the player's score      
       Frame_Count[Player_frame+player_offset]|=((Pin_count[1]&0x0F)<<4);
       print_score(player);

       //do we have enough to throw again?
	   if((Pin_count[0]+Pin_count[1])>=10)
       {
        Player_turn=0;
        Pin_count[0]=0;
        Pin_count[1]=0;
        draw_pins(); //new frame, redraw the pins
        Player_frame++;
	   }
       else
       {
	    print_score(player);
        //reset the scrore for the frame
        Pin_count[0]=0;
        Pin_count[1]=0;
        Player_turn=0;

	    player++; //go to next player
	    //go back to first player, advance a frame
	    if(player>max_players)
	    {
	     print_string(5,12,"GAME OVER");
         game_state=3; //last player, end the game
         player=0;
        }
		else
		 draw_pins(); //new frame, redraw the pins   
       }
	  }
      else if(Player_turn==0 && Player_frame==10) //3rd part of 10th frame
	  {
       //update score box
       draw_frame_score(player, Player_turn);
	   Frame_Count[Player_frame+player_offset]=Pin_count[0];
	   print_score(player);
       Player_frame--; // do this because this a third throw more than the next frame
       //reset the scrore for the frame
       Pin_count[0]=0;
       Pin_count[1]=0;
       Player_turn=0;
	   
       player++; //go to next player
	   //go back to first player, advance a frame
	   if(player>max_players)
	   {
	     print_string(5,12,"GAME OVER");
         game_state=3; //last player, end the game
         player=0;
       }   
		else
		 draw_pins(); //new frame, redraw the pins   
 	  }
     }     
}

//check adjacent pins, knock them out
void check_pin_adj(int x, int y)
{
 int Turn;
 Turn=Player_turn;
 if(Player_turn>1)
  Turn=0;
 if(g_TileMap[(y-1)*SCREEN_TILE_MAP_WIDTH+(x+1)]==2)
 {
    g_TileMap[(y-1)*SCREEN_TILE_MAP_WIDTH+(x+1)]=0;
    //add to score
    Pin_count[Turn]++;
 }
 if(g_TileMap[(y+1)*SCREEN_TILE_MAP_WIDTH+(x+1)]==2)
 {
    g_TileMap[(y+1)*SCREEN_TILE_MAP_WIDTH+(x+1)]=0;
    //add to score
    Pin_count[Turn]++;
 }
}


void check_pin(int x, int y)
{
 int Turn;
 Turn=Player_turn;

          //check hit against the pins
          if(g_TileMap[y*SCREEN_TILE_MAP_WIDTH+x]==2)
          {
           //if we hit pins check pins around it
           g_TileMap[(y)*SCREEN_TILE_MAP_WIDTH+(x)]=0;
           //add to score
           Pin_count[Turn]++;
           check_pin_adj(x,y);
           check_pin_adj(x+1,y-1);
           check_pin_adj(x+1,y+1);
           check_pin_adj(x+2,y-2);
           check_pin_adj(x+2,y+2);
          }
}


//draws the pins on the play field
void draw_pins()
{
 int n =0,x,y;
 for (y=0;y<7;y++)
 {
   for(x=0;x<5;x++)
   { 
     g_TileMap[(y+14)*SCREEN_TILE_MAP_WIDTH+(x+15)]=pins[n];
     n++;
   }
 }
}


//start a new game
void new_game()
{
    int i;
    //set new game
    for(i=0;i<11*3;i++)
     Frame_Count[i]=0; //score for frames
    Pin_count[0]=0; //number of pins hit per turn
    Pin_count[1]=0;
    Player_turn=0, //which part of frame player is in
    Player_frame=0; //which frame player is in
    //erase score frame box
    for(i=0;i<10;i++)
	{
     g_TileMap[(2)*SCREEN_TILE_MAP_WIDTH+(i*2+1)]=0;
     g_TileMap[(3)*SCREEN_TILE_MAP_WIDTH+(i*2+1)]=0;
	}
   	//setup ball
    ball_x=1;
    ball_y=Lane_Start;
    ball_x_old=ball_x;
    ball_y_old=ball_y;
    ball_speed_counter=ball_move_speed;
    ball_dir=0;
}

//draws the title screen
void draw_title_screen()
{
   int n;
   //clear screen
   fill_screen(0);
   print_string(4, 6,"XGS BOWLING!");
   print_string(2,10,"SELECT NUMBER OF");
   print_string(2,11,"    PLAYERS");
   print_string(4,13," 1 PLAYER");
   print_string(4,14," 2 PLAYERS");
   print_string(4,15," 3 PLAYERS");
   print_string(4,22,"  JT COOK");
   for(n=0;n<SCREEN_TILE_MAP_WIDTH;n++)
   {
    g_TileMap[0*SCREEN_TILE_MAP_WIDTH+n]=2;
    g_TileMap[(SCREEN_TILE_MAP_HEIGHT*SCREEN_TILE_MAP_WIDTH)-SCREEN_TILE_MAP_WIDTH+n]=2;
   }

   for(n=0;n<SCREEN_TILE_MAP_HEIGHT;n++)
   {
    g_TileMap[n*SCREEN_TILE_MAP_WIDTH]=2;
    g_TileMap[n*SCREEN_TILE_MAP_WIDTH+(SCREEN_TILE_MAP_WIDTH-1)]=2;
   }
}

//draw the game play screen
void draw_game_screen(int max_players)
{
    int i,n;
    //clear the screen
    fill_screen(0);
    //draw the lanes
    for(i=0;i<20;i++)
    {
     g_TileMap[12*SCREEN_TILE_MAP_WIDTH+i]=1; 
     g_TileMap[22*SCREEN_TILE_MAP_WIDTH+i]=1; 
    }
    //draw frame numbers
    for(i=0;i<10;i++)
    {
     print_char(1+(i<<1),0,i+49); 
    }
    //draw 10 for the last frame
    print_char(18,0,49);
    print_char(19,0,48);
    //draw the score box
    for(i=0;i<20;i++)
    {
     g_TileMap[1*SCREEN_TILE_MAP_WIDTH+i]=4; 
     //g_TileMap[4*SCREEN_TILE_MAP_WIDTH+i]=4; 
     for(n=0;n<max_players+1;n++)
      g_TileMap[(1+(n*3))*SCREEN_TILE_MAP_WIDTH+i]=4; 
    }
    //draw columns
    for(i=0;i<10;i++)
    {
     for(n=0;n<(3*max_players);n++)
      g_TileMap[(n+1)*SCREEN_TILE_MAP_WIDTH+i*2]=4; 
    }
    draw_pins();
    for(n=0;n<max_players;n++)
     print_score(n);
}

void draw_current_player(int player, char on_off)
{
	   char game_text[20];
	   if(on_off==0)
       {
        g_TileMap[(11*SCREEN_TILE_MAP_WIDTH)+0+(7*player)]=0; 
        g_TileMap[(11*SCREEN_TILE_MAP_WIDTH)+1+(7*player)]=0; 
        g_TileMap[(11*SCREEN_TILE_MAP_WIDTH)+2+(7*player)]=0; 
	   }
	   else
	   {
        sprintf(game_text,"P%i:",player+1);
        print_string(7*player, 11, game_text);
	   }
}


void draw_frame_score(int player, int throw)
{
	int player_offset=player*3;
    //first throw
	if(throw==0 && Player_frame<10) 
	{
        if(Pin_count[0]==0)
         print_char((Player_frame*2+1),2+player_offset,45); //draw a dash for 0
        else if(Pin_count[0]==10)
      	 print_char((Player_frame*2+1),2+player_offset,88); //draw a strike
		else
         print_char((Player_frame*2+1),2+player_offset,Pin_count[0]+48); //draw the number
	}
    //third throw, 10th frame
	else if(throw==0 && Player_frame==10) 
	{
        if(Pin_count[0]==0)
         print_char((Player_frame*2+1-3),3+player_offset,45); //draw a dash for 0
        else if(Pin_count[0]==10)
      	 print_char((Player_frame*2+1-3),3+player_offset,88); //draw a strike
		else
         print_char((Player_frame*2+1-3),3+player_offset,Pin_count[0]+48); //draw the number
	}
    //second throw
	else 
	{
       if(Pin_count[1]==10 && Player_frame==9)//are we in the 10th frame?
        print_char((Player_frame*2+1),3+player_offset,88); //draw a strike
       //if it isn't a strike, draw the frame score normally
       else
       {
		if(Pin_count[0]==10 && Player_frame<9) //threw a strike
       	 print_char((Player_frame*2+1),3+player_offset,32); //draw a blank for a strike
        else if(Pin_count[1]==0)
         print_char((Player_frame*2+1),3+player_offset,45); //draw a dash for 0
        else if(Pin_count[1]+Pin_count[0]==10)
		 print_char((Player_frame*2+1),3+player_offset,47); //draw a spare
        else
         print_char((Player_frame*2+1),3+player_offset,Pin_count[1]+48); //draw the number
       }
	}
}

void print_score(int player)
{
	   char game_text[20];
       int Player_score;
       Player_score=Add_score(Frame_Count+(player*11));
       sprintf(game_text,"P%i:%i",player+1,Player_score);
       print_string(7*player, 11, game_text);
}

int Add_score(char *g_score)
{
 //add each frame, a strike adds the next 2 balls, a spare adds the next ball
 int total_score=0;
 char ball1,ball2,ball3;
 int frame_counter=0;
 int n;
 //Ball 1 - bits 0-3
 //Ball 2 - bits 4-7
 for (n=0;n<10;n++)
 {
  ball1=(g_score[n])&0x0F; //score for ball 1
  ball2=(g_score[n]&0xF0)>>4; //score for ball 2
  //strike or spare, add next ball, add second ball for strike
  if(ball1+ball2==10)
  {
   if((n+1)<11)
   {
    ball3=(g_score[n+1])&0x0F;
    total_score+=ball3;
    if(ball1==10) //strike
    {
     if(ball3==10 && (n+2)<11) //next ball also a strike
          total_score+=(g_score[n+2])&0x0F;
	 else if(ball3<10) //if next ball is not strike, add in next ball
          total_score+=(g_score[n+1]&0xF0)>>4;
    }
   }
  }
  total_score+=ball1;
  total_score+=ball2;
 }
 //add in 3rd ball in 10th frame
 total_score+=g_score[10]&0x0F;
 return total_score;
}

void animate_bowler()
{
	int animate_delay, sprite_offset;
	int n;
    WAIT_FOR_VSYNC_END();
	for(n=0;n<3;n++)
	{
	 if(n<2)
	  animate_delay=10;
	 else
	  animate_delay=0;

     sprite_offset=(64*4*n);//which sprite
	 //sprite_offset=0;
	 g_Sprites[1].PixelData = &g_SpriteData[64+sprite_offset];
	 g_Sprites[2].PixelData = &g_SpriteData[(64*2)+sprite_offset];
	 g_Sprites[3].PixelData = &g_SpriteData[(64*3)+sprite_offset];
	 g_Sprites[4].PixelData = &g_SpriteData[(64*4)+sprite_offset];

	 while(animate_delay>1)
	 {
        WAIT_FOR_VSYNC_START();
		animate_delay--;
        WAIT_FOR_VSYNC_END();
	 }
	}
}